@@ -29,6 +29,62 @@
 
 (in-package :cl-unicode)
 
+(defun try-abbreviations (name scripts-to-try)
+  "Helper function called by CHARACTER-NAMED when the
+:TRY-ABBREVIATIONS-P keyword argument is true.  Tries to interpret
+NAME as an abbreviation for a longer Unicode name and returns the
+corresponding code point if it succeeds."
+  (flet ((size-word (string)
+           (if (ppcre:scan "[A-Z]" string) "CAPITAL" "SMALL"))
+         (try (script size-word short-name)
+           (or (character-named (format nil "~A ~A letter ~A"
+                                        script size-word short-name)
+                                :want-code-point-p t)
+               (character-named (format nil "~A letter ~A"
+                                        script short-name)
+                                :want-code-point-p t)
+               (character-named (format nil "~A ~A"
+                                        script short-name)
+                                :want-code-point-p t))))
+    (ppcre:register-groups-bind (script short-name)
+        ("^([^:]+):([^:]+)$" name)
+      (let ((size-word (size-word short-name)))
+        (return-from try-abbreviations
+          (try script size-word short-name))))
+    (loop with size-word = (size-word name)
+          for script in scripts-to-try
+          thereis (try script size-word name))))  
+
+(defun unicode-name-reader (stream char arg)
+  "The reader function used when the alternative character syntax is
+enabled."
+  (declare (ignore char arg))
+  (let ((name (with-output-to-string (out)
+                (write-char (read-char stream t nil t) out)
+                (loop for next-char = (read-char stream t nil t)
+                      while (find next-char "abcdefghijklmnopqrstuvwxyz0123456789_-+:"
+                                  :test 'char-equal)
+                      do (write-char next-char out)
+                      finally (unread-char next-char stream)))))
+    (or (character-named name)
+        (error 'character-not-found :name name))))
+
+(defun %enable-alternative-character-syntax ()
+  "Internal function used to enable alternative character syntax and
+store current readtable on stack."
+  (push *readtable* *previous-readtables*)
+  (setq *readtable* (copy-readtable))
+  (set-dispatch-macro-character #\# #\\ 'unicode-name-reader)
+  (values))
+
+(defun %disable-alternative-character-syntax ()
+  "Internal function used to restore previous readtable." 
+  (setq *readtable*
+        (if *previous-readtables*
+          (pop *previous-readtables*)
+          (copy-readtable nil)))
+  (values))
+
 (defgeneric unicode-name (c)
   (:documentation "Returns the Unicode name of a character as a string
 or NIL if there is no name for that particular character.  C can be
